#include <QDateTime>
#include <QMessageBox>

#include "mainwindow.h"
#include "ui_mainwindow.h"

#include "caboutdialog.h"
#include "csettingsdialog.h"

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow),
    m_iErrorCount(0),
    m_iRequestCount(0),
    m_iByteCount(0)
{
    ui->setupUi(this);

    // Now initialize the server module

    m_server = new CHTTPServer();

    // Now connect to the signals

    connect(m_server,SIGNAL(statusStarted()),this,SLOT(OnStarted()));
    connect(m_server,SIGNAL(statusStopped()),this,SLOT(OnStopped()));
    connect(m_server,SIGNAL(statusString(QString)),this,SLOT(OnString(QString)));
    connect(m_server,SIGNAL(statusInfo(int)),this,SLOT(OnInfo(int)));
    connect(m_server,SIGNAL(statusError()),this,SLOT(OnError()));
}

MainWindow::~MainWindow()
{
    if(m_server->isRunning())
    {
        m_server->Abort();
        m_server->wait();
    }

    delete m_server;

    delete ui;
}

void MainWindow::OnExit()
{
    close();
}

void MainWindow::OnAbout()
{
    CAboutDialog * pAbout = new CAboutDialog(this);
    pAbout->exec();
}

void MainWindow::OnSettings()
{
    CSettingsDialog * pSettings = new CSettingsDialog(this);

    // If any of the settings were changed
    // we'll need to let the user
    // know that the server needs restarting
    if(pSettings->exec() == QDialog::Accepted)
    {
        QMessageBox::warning(this,"Warning:","The server needs to be restarted before any changes can take effect.");
    }
}

void MainWindow::OnStart()
{
    // Start the server!!!

    m_server->start();
}

void MainWindow::OnStop()
{
    // Stop the server

    m_server->Abort();
}

void MainWindow::OnStarted()
{
    ui->start_server->setEnabled(false);
    ui->stop_server->setEnabled(true);

    ui->label_status_text->setStyleSheet("color: rgb(25,170,25);");
    ui->label_status_text->setText("Running");
}

void MainWindow::OnStopped()
{
    ui->start_server->setEnabled(true);
    ui->stop_server->setEnabled(false);

    ui->label_status_text->setStyleSheet("color: rgb(255,170,0);");
    ui->label_status_text->setText("Stopped");
}

void MainWindow::OnString(QString str)
{
    // Figure out the current date / time

    ui->console->appendPlainText(tr("[%1] %2")
                                 .arg(QDateTime::currentDateTime().toString("MM/dd/yyyy hh:mm:ss"))
                                 .arg(str));
}

void MainWindow::OnInfo(int iBytes)
{
    m_iRequestCount += 1;
    m_iByteCount += iBytes;

    ui->label_pages_text->setText(tr("%1").arg(m_iRequestCount));
    ui->label_bytes_text->setText(tr("%1").arg(m_iByteCount));
}

void MainWindow::OnError()
{
    if(!m_iErrorCount)
        ui->label_errors_text->setStyleSheet("color: rgb(170, 25, 25);");

    m_iErrorCount += 1;

    ui->label_errors_text->setText(tr("%1").arg(m_iErrorCount));
}

void MainWindow::changeEvent(QEvent *e)
{
    QMainWindow::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}
